//
// $Id: search.cc,v 1.2 2004/01/24 20:26:17 ejb Exp $
// $Source: /home/ejb/Q/pstt/CVSroot/test-coverage/example/search/search.cc,v $
// $Author: ejb $
//


#ifndef NO_RCS_HDRS
// Define a static function and call it.  No warnings this way.
static void rcsid(const char * const)
{rcsid("@(#)$Id: search.cc,v 1.2 2004/01/24 20:26:17 ejb Exp $");}
#endif // ! NO_RCS_HDRS

#include <iostream>
#include <stdlib.h>
#include <vector>
#include "TCOV.hh"

#ifdef CRNL
static char const* TESTNL = "\r\n";
#else
static char const* TESTNL = "\n";
#endif

// If you change this threshold to something greater than 15 or less
// than 4, the test suite passes but coverage fails.
static unsigned int const SCAN_THRESHOLD = 10;

// Find an item by doing a linear scan
int scan(std::vector<int> const& v, int to_find)
{
    int nitems = v.size();
    for (int i = 0; i < nitems; ++i)
    {
	if (v[i] == to_find)
	{
	    TCOV::TC("search", "linear scan found");
	    return i;
	}
    }
    TCOV::TC("search", "linear scan not found",
	     ((to_find < v[0]) ? 0 :		// too low
	      (to_find > v.back()) ? 1 :	// too high
	      2));				// just not there
    return -1;
}

// Find the item with a binary search
int bsearch(std::vector<int> const& v, int to_find)
{
    int low = 0;
    int high = v.size() - 1;
    while (low <= high)
    {
	int test = (low + high) / 2;
	if (v[test] == to_find)
	{
	    TCOV::TC("search", "bsearch found");
	    return test;
	}
	else if (v[test] < to_find)
	{
	    TCOV::TC("search", "bsearch increased low");
	    low = test + 1;
	}
	else
	{
	    TCOV::TC("search", "bsearch decreased high");
	    high = test - 1;
	}
    }
    TCOV::TC("search", "bsearch not found",
	     ((to_find < v[0]) ? 0 :		// too low
	      (to_find > v.back()) ? 1 :	// too high
	      2));				// just not there
    return -1;
}

// Call one of scan() or bsearch() based on the size of the vector
int search(std::vector<int> const& v, int to_find)
{
    if (v.size() <= SCAN_THRESHOLD)
    {
	return scan(v, to_find);
    }
    else
    {
	return bsearch(v, to_find);
    }
}

int main(int argc, char* argv[])
{
    if (argc != 3)
    {
	std::cerr << "Usage: search n-items to-find" << TESTNL;
	exit(2);
    }
    int nitems = atoi(argv[1]);
    if (nitems < 1)
    {
	TCOV::TC("search", "nitems < 1");
	std::cerr << "search: n-items must be >= 1" << TESTNL;
	exit(2);
    }
    int to_find = atoi(argv[2]);
    std::vector<int> v;
    for (int i = 0; i < nitems; ++i)
    {
	v.push_back(i * 3);
    }
    int idx = search(v, to_find);
    TCOV::TC("search", "idx location",
	     ((idx == -1) ? 0 :			// not found
	      (idx == 0) ? 1 :			// first item
	      (idx < nitems - 1) ? 2 :		// somewhere in the middle
	      (idx == nitems - 1) ? 3 :		// last item
	      4));				// can't happen
    std::cout << "index: " << idx << TESTNL;
    return 0;
}
